@@ -9,12 +9,15 @@
 #include "../../lib/curlx/fopen.c"
 #include "../../lib/curlx/multibyte.c"
 #include "../../lib/curlx/strerr.c"
+#include "../../lib/curlx/strparse.c"
 #include "../../lib/curlx/timediff.c"
 #include "../../lib/curlx/timeval.c"
 #include "../../lib/curlx/version_win32.c"
 #include "../../lib/curlx/wait.c"
 #include "../../lib/curlx/warnless.c"
 #include "../../lib/curlx/winapi.c"
+#include "../../src/toolx/tool_time.c"
+#include "cli_ftp_upload.c"
 #include "cli_h2_pausing.c"
 #include "cli_h2_serverpush.c"
 #include "cli_h2_upgrade_extreme.c"
@@ -179,6 +182,7 @@
 #include "lib1569.c"
 #include "lib1571.c"
 #include "lib1576.c"
+#include "lib1582.c"
 #include "lib1591.c"
 #include "lib1592.c"
 #include "lib1593.c"
@@ -260,6 +264,7 @@
 #include "lib1521.c"
 
 const struct entry_s s_entries[] = {
+  {"cli_ftp_upload", test_cli_ftp_upload},
   {"cli_h2_pausing", test_cli_h2_pausing},
   {"cli_h2_serverpush", test_cli_h2_serverpush},
   {"cli_h2_upgrade_extreme", test_cli_h2_upgrade_extreme},
@@ -424,6 +429,7 @@ const struct entry_s s_entries[] = {
   {"lib1569", test_lib1569},
   {"lib1571", test_lib1571},
   {"lib1576", test_lib1576},
+  {"lib1582", test_lib1582},
   {"lib1591", test_lib1591},
   {"lib1592", test_lib1592},
   {"lib1593", test_lib1593},
